[PATCH] D122601: Fix MemorySSAUpdater::insertDef for dead code

Artur Pilipenko via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Thu Mar 31 16:33:24 PDT 2022


This revision was landed with ongoing or failed builds.
This revision was automatically updated to reflect the committed changes.
Closed by commit rG4fbde1ef40e6: Fix MemorySSAUpdater::insertDef for dead code (authored by apilipenko).

Changed prior to commit:
  https://reviews.llvm.org/D122601?vs=418614&id=419589#toc

Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D122601/new/

https://reviews.llvm.org/D122601

Files:
  llvm/lib/Analysis/MemorySSAUpdater.cpp
  llvm/test/Transforms/GVN/mssa-update-dead-def.ll


Index: llvm/test/Transforms/GVN/mssa-update-dead-def.ll
===================================================================
--- /dev/null
+++ llvm/test/Transforms/GVN/mssa-update-dead-def.ll
@@ -0,0 +1,33 @@
+; RUN: opt -passes='require<memoryssa>,gvn' -verify-memoryssa -S %s | FileCheck %s
+
+; This is a regression test for a bug in MemorySSA updater.
+; Make sure that we don't crash and end up with a valid MemorySSA.
+
+; CHECK: @test()
+define void @test() personality i32* ()* null {
+  invoke void @bar()
+          to label %bar.normal unwind label %exceptional
+
+bar.normal:
+  ret void
+
+dead.block:
+  br label %baz.invoke
+
+baz.invoke:
+  invoke void @baz()
+          to label %baz.normal unwind label %exceptional
+
+baz.normal:
+  ret void
+
+exceptional:
+  %tmp9 = landingpad { i8*, i32 }
+          cleanup
+  call void @foo()
+  ret void
+}
+
+declare void @foo()
+declare void @bar()
+declare void @baz()
Index: llvm/lib/Analysis/MemorySSAUpdater.cpp
===================================================================
--- llvm/lib/Analysis/MemorySSAUpdater.cpp
+++ llvm/lib/Analysis/MemorySSAUpdater.cpp
@@ -305,6 +305,12 @@
 // point to the correct new defs, to ensure we only have one variable, and no
 // disconnected stores.
 void MemorySSAUpdater::insertDef(MemoryDef *MD, bool RenameUses) {
+  // Don't bother updating dead code.
+  if (!MSSA->DT->isReachableFromEntry(MD->getBlock())) {
+    MD->setDefiningAccess(MSSA->getLiveOnEntryDef());
+    return;
+  }
+
   VisitedBlocks.clear();
   InsertedPHIs.clear();
 
@@ -422,10 +428,10 @@
   if (NewPhiSize)
     tryRemoveTrivialPhis(ArrayRef<WeakVH>(&InsertedPHIs[NewPhiIndex], NewPhiSize));
 
   // Now that all fixups are done, rename all uses if we are asked. Skip
   // renaming for defs in unreachable blocks.
   BasicBlock *StartBlock = MD->getBlock();
-  if (RenameUses && MSSA->getDomTree().getNode(StartBlock)) {
+  if (RenameUses) {
     SmallPtrSet<BasicBlock *, 16> Visited;
     // We are guaranteed there is a def in the block, because we just got it
     // handed to us in this function.


-------------- next part --------------
@@ -422,10 +428,10 @@
     tryRemoveTrivialPhis(ArrayRef<WeakVH>(&InsertedPHIs[NewPhiIndex], NewPhiSize));
 
-  // Now that all fixups are done, rename all uses if we are asked. Skip
-  // renaming for defs in unreachable blocks.
+  // Now that all fixups are done, rename all uses if we are asked. The defs are
+  // guaranteed to be in reachable code due to the check at the method entry.
   BasicBlock *StartBlock = MD->getBlock();
A non-text attachment was scrubbed...
Name: D122601.419589.patch
Type: text/x-patch
Size: 2247 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20220331/adaf7da0/attachment.bin>


More information about the llvm-commits mailing list